@@ -52,7 +52,10 @@ class Writer:
 
         # 2) If IDs must be returned, use existing execute_multi_query path
         if save_ids:
-            args = [tuple(None if pd.isna(val) else t(val) for t, val in zip(self.TYPES, row, strict=False)) for row in df.values]
+            args = [
+                tuple(None if pd.isna(val) else t(val) for t, val in zip(self.TYPES, row, strict=False))
+                for row in df.values
+            ]
             query = self.build_query(single_placeholder=False)
             ids = self.db.execute_multi_query(
                 query, args, fetchone=True, commit=True, show_progress_bar=show_progress_bar
