@@ -5,14 +5,22 @@ from kedro.pipeline.node import Node
 
 from .dataloading_nodes import (
     check_data_inputs,
+    node_sync_regions,
     node_write_categories,
     node_write_company,
-    node_write_datapoints,
+    node_write_flags,
     node_write_product_categories,
+    node_write_product_data,
     node_write_products,
+    node_write_sales,
+    node_write_sku_data,
     node_write_skus,
+    node_write_store_data,
     node_write_stores,
+    node_write_time_product_data,
+    node_write_time_region_data,
     node_write_time_sku_data,
+    node_write_time_store_data,
 )
 
 # node_write_categories,; node_write_datapoints,; node_write_flags,; node_write_product_categories,; node_write_product_data,; node_write_products,; node_write_sku_data,; node_write_skus,; node_write_store_data,; node_write_stores,; node_write_time_product_data,; node_write_time_region_data,; node_write_time_sku_data,; node_write_time_store_data,
@@ -21,6 +29,16 @@ from .dataloading_nodes import (
 def create_writing_sub_pipeline() -> list[Node]:
     sub_pipeline = [
         #################################### Data input validation ########################################
+        node(
+            func=node_sync_regions,
+            inputs={
+                "data_loader": "data_loader",
+                "check_passed": "check_passed",
+            },
+            outputs="regions_synced",
+            name="sync_regions",
+        ),
+        
         node(
             func=check_data_inputs,
             inputs=[
@@ -33,8 +51,7 @@ def create_writing_sub_pipeline() -> list[Node]:
                 "categories_dict",
                 "categories_level_description",
                 "products",
-                "time_sku_data",
-                "time_sku_feature_description_map",
+                "sales_data_map",
                 "flags",
                 "store_feature_description_map",
                 "store_feature_map",
@@ -48,6 +65,9 @@ def create_writing_sub_pipeline() -> list[Node]:
                 "time_region_feature_map",
                 "time_store_feature_description_map",
                 "time_store_feature_map",
+                "time_sku_data",
+                "time_sku_feature_description_map",
+                "regions_synced"
             ],
             outputs="check_passed",
             name="check_data_inputs",
@@ -64,6 +84,7 @@ def create_writing_sub_pipeline() -> list[Node]:
                 "max_date": "max_date",
                 "frequency": "frequency",
                 "check_passed": "check_passed",
+                "regions_synced": "regions_synced",
             },
             outputs="company_id",
             name="write_company",
@@ -121,21 +142,126 @@ def create_writing_sub_pipeline() -> list[Node]:
                 "time_sku_data": "time_sku_data",
                 "store_ids": "store_ids",
                 "product_ids": "product_ids",
+                "company_id": "company_id",
                 "check_passed": "check_passed",
             },
             outputs="sku_ids",
             name="write_skus",
         ),
         node(
-            func=node_write_datapoints,
+            func=node_write_sales,
             inputs={
                 "data_loader": "data_loader",
-                "time_sku_data": "time_sku_data",
+                "sales": "sales_data_map",
                 "sku_ids": "sku_ids",
+                "company_id": "company_id",
                 "check_passed": "check_passed",
+                "num_workers": "params:num_workers",
+                "time_data_strategy": "params:time_data_strategy",
             },
-            outputs="datapoint_ids",
-            name="write_datapoints",
+            outputs="sales_written",
+            name="write_sales",
+        ),
+        node(
+            func=node_write_flags,
+            inputs={
+                "data_loader": "data_loader",
+                "flags": "flags",
+                "sku_ids": "sku_ids",
+                "company_id": "company_id",
+                "check_passed": "check_passed",
+                "num_workers": "params:num_workers",
+                "time_data_strategy": "params:time_data_strategy",
+            },
+            outputs="flags_written",
+            name="write_flags",
+        ),
+        #################################### Write optional data ########################################
+        node(
+            func=node_write_store_data,
+            inputs={
+                "data_loader": "data_loader",
+                "store_feature_description_map": "store_feature_description_map",
+                "store_feature_map": "store_feature_map",
+                "store_ids": "store_ids",
+                "company_id": "company_id",
+                "check_passed": "check_passed",
+                "num_workers": "params:num_workers",
+            },
+            outputs="store_features_written",
+            name="write_store_data",
+        ),
+        node(
+            func=node_write_product_data,
+            inputs={
+                "data_loader": "data_loader",
+                "product_feature_description_map": "product_feature_description_map",
+                "product_feature_map": "product_feature_map",
+                "product_ids": "product_ids",
+                "company_id": "company_id",
+                "check_passed": "check_passed",
+                "num_workers": "params:num_workers",
+            },
+            outputs="product_features_written",
+            name="write_product_data",
+        ),
+        node(
+            func=node_write_sku_data,
+            inputs={
+                "data_loader": "data_loader",
+                "sku_feature_description_map": "sku_feature_description_map",
+                "sku_feature_map": "sku_feature_map",
+                "sku_ids": "sku_ids",
+                "company_id": "company_id",
+                "check_passed": "check_passed",
+                "num_workers": "params:num_workers",
+            },
+            outputs="sku_features_written",
+            name="write_sku_data",
+        ),
+        node(
+            func=node_write_time_product_data,
+            inputs={
+                "data_loader": "data_loader",
+                "time_product_feature_description_map": "time_product_feature_description_map",
+                "time_product_feature_map": "time_product_feature_map",
+                "product_ids": "product_ids",
+                "company_id": "company_id",
+                "check_passed": "check_passed",
+                "num_workers": "params:num_workers",
+                "time_data_strategy": "params:time_data_strategy",
+            },
+            outputs="time_product_features_written",
+            name="write_time_product_data",
+        ),
+        node(
+            func=node_write_time_store_data,
+            inputs={
+                "data_loader": "data_loader",
+                "time_store_feature_description_map": "time_store_feature_description_map",
+                "time_store_feature_map": "time_store_feature_map",
+                "store_ids": "store_ids",
+                "company_id": "company_id",
+                "check_passed": "check_passed",
+                "num_workers": "params:num_workers",
+                "time_data_strategy": "params:time_data_strategy",
+            },
+            outputs="time_store_features_written",
+            name="write_time_store_data",
+        ),
+        node(
+            func=node_write_time_region_data,
+            inputs={
+                "data_loader": "data_loader",
+                "time_region_feature_description_map": "time_region_feature_description_map",
+                "time_region_feature_map": "time_region_feature_map",
+                "company_id": "company_id",
+                "check_passed": "check_passed",
+                "num_workers": "params:num_workers",
+                "time_data_strategy": "params:time_data_strategy",
+            },
+            outputs="time_region_features_written",
+            name="write_time_region_data",
         ),
         node(
             func=node_write_time_sku_data,
@@ -143,103 +269,15 @@ def create_writing_sub_pipeline() -> list[Node]:
                 "data_loader": "data_loader",
                 "time_sku_feature_description_map": "time_sku_feature_description_map",
                 "time_sku_data": "time_sku_data",
-                "datapoint_ids": "datapoint_ids",
+                "sku_ids": "sku_ids",
                 "company_id": "company_id",
                 "check_passed": "check_passed",
+                "num_workers": "params:num_workers",
+                "time_data_strategy": "params:time_data_strategy",
             },
-            outputs="time_sku_data_object",
+            outputs="time_sku_data_written",
             name="write_time_sku_data",
         ),
-        # node(
-        #     func=node_write_flags,
-        #     inputs={
-        #         "data_loader": "data_loader",
-        #         "flags": "flags",
-        #         "company_object": "company_object",
-        #         "time_sku_data_with_datapoint_id": "time_sku_data_with_datapoint_id",
-        #         "check_passed": "check_passed",
-        #     },
-        #     outputs="flags_object",
-        #     name="write_flags",
-        # ),
-        # #################################### Write optional data ########################################
-        # node(
-        #     func=node_write_store_data,
-        #     inputs={
-        #         "data_loader": "data_loader",
-        #         "store_feature_description_map": "store_feature_description_map",
-        #         "store_feature_map": "store_feature_map",
-        #         "store_object": "store_object",
-        #         "company_object": "company_object",
-        #         "check_passed": "check_passed",
-        #     },
-        #     outputs=["store_description_object", "store_feature_object"],
-        #     name="write_store_data",
-        # ),
-        # node(
-        #     func=node_write_product_data,
-        #     inputs={
-        #         "data_loader": "data_loader",
-        #         "product_feature_description_map": "product_feature_description_map",
-        #         "product_feature_map": "product_feature_map",
-        #         "product_object": "product_object",
-        #         "company_object": "company_object",
-        #         "check_passed": "check_passed",
-        #     },
-        #     outputs=["product_description_object", "product_feature_object"],
-        #     name="write_product_data",
-        # ),
-        # node(
-        #     func=node_write_sku_data,
-        #     inputs={
-        #         "data_loader": "data_loader",
-        #         "sku_feature_description_map": "sku_feature_description_map",
-        #         "sku_feature_map": "sku_feature_map",
-        #         "sku_object": "sku_object",
-        #         "company_object": "company_object",
-        #         "check_passed": "check_passed",
-        #     },
-        #     outputs=["sku_description_object", "sku_feature_object"],
-        #     name="write_sku_data",
-        # ),
-        # node(
-        #     func=node_write_time_product_data,
-        #     inputs={
-        #         "data_loader": "data_loader",
-        #         "time_product_feature_description_map": "time_product_feature_description_map",
-        #         "time_product_feature_map": "time_product_feature_map",
-        #         "product_object": "product_object",
-        #         "company_object": "company_object",
-        #         "check_passed": "check_passed",
-        #     },
-        #     outputs=["time_product_description_object", "time_product_feature_object"],
-        #     name="write_time_product_data",
-        # ),
-        # node(
-        #     func=node_write_time_region_data,
-        #     inputs={
-        #         "data_loader": "data_loader",
-        #         "time_region_feature_description_map": "time_region_feature_description_map",
-        #         "time_region_feature_map": "time_region_feature_map",
-        #         "company_object": "company_object",
-        #         "check_passed": "check_passed",
-        #     },
-        #     outputs=["time_region_description_object", "time_region_feature_object"],
-        #     name="write_time_region_data",
-        # ),
-        # node(
-        #     func=node_write_time_store_data,
-        #     inputs={
-        #         "data_loader": "data_loader",
-        #         "time_store_feature_description_map": "time_store_feature_description_map",
-        #         "time_store_feature_map": "time_store_feature_map",
-        #         "store_object": "store_object",
-        #         "company_object": "company_object",
-        #         "check_passed": "check_passed",
-        #     },
-        #     outputs=["time_store_description_object", "time_store_feature_object"],
-        #     name="write_time_store_data",
-        # ),
     ]
 
     return sub_pipeline
